/*
 * lhuff.h - simple Huffman coding and other routines for storing data
 *
 * Copyright (c) 2008, Petteri Kaski, Patric Ostergard, Olli Pottonen
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef LHUFF_H
#define LHUFF_H

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

/********************************************* Input / output bit at a time  */

typedef       struct bb_in_s  bb_in;
typedef       struct bb_out_s bb_out;

void            init_bb_in     (bb_in *,  FILE *);
void            init_bb_out    (bb_out *, FILE *);
void            flush_bb_out   (bb_out *);
void            flush_bb_in    (bb_in *);
void            bb_in_seek     (bb_in *, long);
void            bb_out_seek    (bb_out *,long);
long            bb_in_pos      (const bb_in *);
long            bb_out_pos     (const bb_out *);

/******************************************* Input / output a word at a time */

void            writeint       (bb_out *, uint32_t);
uint32_t        readint        (bb_in *);

/************************************************************ Huffman coding */

typedef       struct h_code   huffman_code;

huffman_code  * init_code      (const char *const *codewords);
void            free_code      (huffman_code *);
void            encode_symbol  (const huffman_code*, bb_out*, int);
int             decode_symbol  (const huffman_code*, bb_in*);


void    stsc_error(const char *msg);


/*************************************************** Internal implementation */

/* A byte is considered as a string of bits, starting with the least
 * significant one. */
struct bb_in_s
{
    FILE *         f;
    unsigned char  bits;
    unsigned int   bitcount;
};

struct bb_out_s
{
    FILE *         f;
    unsigned char  bits;
    unsigned int   bitcount;
};

#endif // LHUFF_H
